// Import required modules
import mongoose from 'mongoose'
import UserModel from '../models/users';
import * as dotenv from 'dotenv';

// Load the environment variables from the .env file
dotenv.config();
const url = process.env.MONGODB_URI;

if (!url) {
    throw new Error('MONGODB_URI is not defined in the .env file.');
}

// Connection function to mongoDB database
const dbconnect = async () =>{
    mongoose.connect(url)
    .then(() =>{
        //successful connect
        return 200;
    })
    .catch((error) => {
        //internal server error
        return 500;
    });
}

// Login function
const loginUser = async (username: string, password: string)=>{
    try{
        const user = await UserModel.findOne({ username });
        if(!user){
            return 404;
        }
        
        if(user.password != password){
            // Unauthorized access
            return 401;
        }
        
        else if(user.password == password){
            // Operation successful
            return 200;
        }
        else{
            // Unknown error
            return 500;
        }
    }
    catch{
        // Unknown error
        return 500;
    }
};

// Add new user to database
const addNewUser = async (email: string, username: string, password: string) =>{
    try {
        // Check is username available
        const user = await UserModel.findOne({ username });
        if(user){
            // Username already in database
            return 409;
        }

        // Check if email is in use 
        const e_check = await UserModel.findOne({ email });
        if(e_check){
            // Email already taken
            return 422
        }

        // Create new user
        UserModel.create({
            email: email,
            username: username,
            password: password
            })
            .then(() => {
              // Operation successful
              return 200;
            })
            .catch((error) => {
              // Internal server error
              return 500;
            });

    } catch(error) {
        // Internal server error
        return 500;
    }
};

// Add an api key and IV to a created user
const addApiKeyToUser = async (username: string, apiKey: { name: string, key: string, iv: string }) => {
    try {
        const user = await UserModel.findOne({ username });
  
        if (!user) {
            //user not found
            return 404;
        }
  
        // Add the new API key to the user's array
        user.apiKeys.push(apiKey);
  
        // Save the updated user document
        await user.save();
        
        //operation successful
        return 200
        
    } catch (error) {
        //internal server error
        return 500;
    }
};

// Find users api keys
const getUserApiKeys =  async (username: string) => {
    try{
        const user = await UserModel.findOne({ username });

        if(!user){
            // User not found
            return 404
        }

        if (!user.apiKeys || user.apiKeys.length === 0) {
            // API keys are empty
            return 400;
        }

        return user.apiKeys
    }
    catch{
        return 500;
    }
};

// Function to change users password
const changePass = async (username: string, old_pass: string, new_pass: string, confirm: string) => {
    try{
        const user = await UserModel.findOne({ username });
  
        if (!user) {
            // User not found
            return 404;
        }

        if(user.password != old_pass){
            // Unauthorized access
            return 401;
        }

        if(new_pass != confirm){
            // Password confirmation mismatch
            return 422
        }

        // Update user password
        user.password = new_pass;

        // Save the updated user document
        await user.save();
        
        // Operation successful
        return 200

    }
    catch{
        // Internal server error
        return 500;
    }
};

export { dbconnect, loginUser, addApiKeyToUser, addNewUser, changePass, getUserApiKeys };
